﻿using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TransportCompanyContracts.BindingModels;
using TransportCompanyContracts.SearchModels;
using TransportCompanyContracts.StoragesContracts;
using TransportCompanyContracts.ViewModels;
using TransportCompanyDatabaseImplements.Models;

namespace TransportCompanyDatabaseImplements.Implements
{
	public class TruckingStorage : ITruckingStorage
	{
		public TruckingViewModel? Delete(TruckingBindingModel model)
		{
			using var context = new ElegevContext();

			var element = context.Truckings
				.FirstOrDefault(rec => rec.Id == model.Id);

			if (element != null)
			{
				context.Truckings.Remove(element);
				context.SaveChanges();

				return element.GetViewModel;
			}

			return null;
		}

		public TruckingViewModel? GetElement(TruckingSearchModel model)
		{
			if (!model.Id.HasValue || !model.ClientId.HasValue)
			{
				return null;
			}

			using var context = new ElegevContext();

			return context.Truckings
				.FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))
				?.GetViewModel;
		}

		public List<TruckingViewModel> GetFilteredList(TruckingSearchModel model)
		{
			if (!model.Id.HasValue)
			{
				return new();
			}

			using var context = new ElegevContext();

			return context.Truckings
				.Where(x => x.Id == model.Id)
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public List<TruckingViewModel> GetFullList()
		{
			using var context = new ElegevContext();

			return context.Truckings
				.Include(x => x.Transport)
				.Include(x => x.Cargo)
				.Include(x => x.Transportation)
				.Include(x => x.Client)
				.Select(x => x.GetViewModel)
				.ToList();
		}

		public TruckingViewModel? Insert(TruckingBindingModel model)
		{
			using var context = new ElegevContext();

			model.Id = context.Truckings.Count() > 0 ? context.Truckings.Max(x => x.Id) + 1 : 1;

			var newTrucking = Trucking.Create(model);

			if (newTrucking == null)
			{
				return null;
			}

			context.Truckings.Add(newTrucking);
			context.SaveChanges();

			return context.Truckings
					.Include(x => x.Transport)
					.Include(x => x.Cargo)
					.Include(x => x.Transportation)
					.Include(x => x.Client)
					.FirstOrDefault(x => x.Id == model.Id)
					?.GetViewModel;
		}

		public TruckingViewModel? Update(TruckingBindingModel model)
		{
			using var context = new ElegevContext();
			using var transaction = context.Database.BeginTransaction();

			try
			{
				var trucking = context.Truckings.FirstOrDefault(rec => rec.Id == model.Id);

				if (trucking == null)
				{
					return null;
				}

				trucking.Update(model);
				context.SaveChanges();
				transaction.Commit();

				return trucking.GetViewModel;
			}
			catch
			{
				transaction.Rollback();
				throw;
			}
		}
	}
}
